/************************************************
* PICKLIST.C - functions to make a 'file pick'	*
* routine (like the one in Turbo C's IDE).		*
************************************************/

#include <stdio.h>		/* fputc, stderr, ...	*/
#include <ctype.h>		/* isupper, toupper, ...*/
#include <conio.h>		/* Color definitions...	*/
#include <dir.h>		/* ffblk, getdisk, ...	*/
#include <string.h>		/* strcpy, strcat, ...	*/
#include <stdlib.h>		/* free, div, div_t, ...*/
#include <dos.h>		/* FA_DIREC				*/
#include "windows.h"
#include "picklist.h"

#define EL_SIZE sizeof(char [14])
#define FALSE   (1==0)
#define TRUE    (1==1)

/*  Global variables.  */
static char (*dname)[14]=NULL, (*sname)[14]=NULL,
		Blanks[]="             ";
int		curr_elem, curr_page, max_elem, max_page,
		max_page_elem;

/************************************************
* pick_list_dflt - default filename processor	*
*												*
* INP:	p - current drive & path				*
*		n - name of file						*
* OUT:	FALSE (i.e. don't quit)					*
* NOTE:	p is assumed to be long enough to hold	*
*		complete pathname of a file.			*
************************************************/
int pick_list_dflt(char *p, char *n)
	{
	strcat(p,"\\");
	strcat(p,n);
	return TRUE;
	}

/************************************************
* pick_list - allow user to select any file		*
*												*
* INP:	file - ptr to buffer to hold filename	*
*		wild - wildcard file specifier			*
*		func - function to process selected file*
*				(if result==TRUE, terminate)	*
* OUT:	TRUE on error, FALSE otherwise			*
* NOTE:	file is modified to hold file path		*
* NOTE: file has to be at least MAXPATH chars	*
************************************************/
int pick_list(char *file, char *wild,
				int (*func)(char *,char *))
{
	int	ch, i, num_drives;

	Screen(SAVE);
	WindowCreate(7,4,72,19,LO_FG,LO_BG);

	num_drives = setdisk(getdisk());
	ch = NULL;
	while (ch != ESC)
	{
		if (!dname)
			if (build_dir_display(wild))
				return TRUE;
		ch = getch();
		if (ch==0) ch=getch()+0x80;
		switch(ch) {
			case PGUPKEY :
				if (--curr_page < 0)
					curr_page++;
				curr_elem = 0;
				display_dir_page(curr_page,wild);
				break;
			case PGDNKEY :
				if (++curr_page > max_page)
					curr_page--;
				curr_elem = 0;
				display_dir_page(curr_page,wild);
				break;
			case HOMEKEY :
				curr_page = 0;
				curr_elem = 0;
				display_dir_page(curr_page,wild);
				break;
			case ENDKEY :
				curr_page = max_page;
				curr_elem = 0;
				display_dir_page(curr_page,wild);
				break;
			case UPKEY :
				i = curr_elem;
				if ((curr_elem -= 4) < 0)
					curr_elem += 4;
				display_elem(i);
				display_elem(curr_elem);
				break;
			case DOWNKEY :
				i = curr_elem;
				if ((curr_elem+=4)>max_page_elem)
					curr_elem -= 4;
				display_elem(i);
				display_elem(curr_elem);
				break;
			case LEFTKEY :
				i = curr_elem;
				if ((curr_elem -= 1) < 0)
					curr_elem += 1;
				display_elem(i);
				display_elem(curr_elem);
				break;
			case RIGHTKEY :
				i = curr_elem;
				if ((curr_elem+=1)>max_page_elem)
					curr_elem -= 1;
				display_elem(i);
				display_elem(curr_elem);
				break;
			case CR:
				if (sname[curr_elem][0]=='\\')
					{
					strtok(sname[curr_elem]," ");
					chdir(sname[curr_elem]+1);
					getcwd(file,MAXPATH);
					free(dname); dname=NULL;
					}
				else switch ((*func)(file,
								sname[curr_elem]))
					{
					case 0:	break;
					case 1: ch=ESC; break;
					case 2: free(dname); dname=NULL;
							break;
					}
				break;
			case ESC:
				file[0]=0;
				break;
			default:
				ch=toupper(ch);
				if (isupper(ch)&&ch<'A'+num_drives)
					{
					setdisk(ch-'A');
					free(dname); dname=NULL;
					}
				else
					fputc('\7',stderr);
				break;
		}
	}

	Screen(RESTORE);
	free(dname);
	dname=NULL;
	return FALSE;
}

/************************************************
* build_dir_display - allocate RAM for and load	*
* RAM with file and subdirectory entries for 	*
* current directory.  Display first page.		*
*												*
* INP:	wild - wildcard specifier for files		*
************************************************/
int build_dir_display(char *wild)
{
	struct	ffblk ffblk;
	int		done;
	div_t	mp;
	int		cur_max;

	done=findfirst(wild,&ffblk,FA_DIREC);
	max_elem=0;
	for (cur_max=0; done==0; done=findnext(&ffblk))
		{
		if (max_elem>=cur_max)
			{
			cur_max += 48;
			dname=(char(*)[14])realloc(dname,
									cur_max*14);
			if (!dname) return TRUE;
			}
		if (!strcmp(ffblk.ff_name,"."))
			continue;
		if (ffblk.ff_attrib & FA_DIREC)
			strcpy(dname[max_elem],"\\");
		else
			dname[max_elem][0]='';
		strcat(dname[max_elem],ffblk.ff_name);
		strcat(dname[max_elem],
			Blanks+strlen(dname[max_elem]));
		++max_elem;
		}

	mp = div(max_elem,48);
	max_page = mp.quot;
	qsort(dname,max_elem,14,strcmp);
	curr_elem = curr_page = 0;
	display_dir_page(curr_page,wild);
	return FALSE;
}


/************************************************
* display_dir_page - show the current page of	*
* the directory in the window					*
*												*
* INP:	curr_page - the current page number		*
************************************************/
void display_dir_page(int curr_page, char *wild)
{
	char *curdir, wbuf[100];
	int   i;

	sname=dname+curr_page*48;
	clrscr();
	curdir=getcwd(NULL,50);
	sprintf(wbuf,"Directory: %s\\%s",curdir,wild);
	free(curdir);
	for (i = 0; i < 48; i++)
		if ((curr_page * 48) + i < max_elem) {
			display_elem(i);
			max_page_elem = i;
		}
	display_elem(curr_elem);
	Windowputs(1,14,wbuf);
}

/************************************************
* display_elem - display an array element in	*
* the appropriate screen location				*
*												*
* INP:	e - the number of the element to show	*
************************************************/
void display_elem(int e)
{
	div_t	rc;

	/*  Highlight the current element.  */
	if (e == curr_elem) {
		textcolor(HI_FG);
		textbackground(HI_BG);
	}
	rc = div(e,4);
	Windowputs((rc.rem*16)+1,rc.quot+1,sname[e]);

	/*  Return to normal display attributes.  */
	if (e == curr_elem) {
		textcolor(LO_FG);
		textbackground(LO_BG);
	}
}
