/************************************************
* WINDOWS.C - text windowing functions			*
************************************************/

#include <conio.h>  	/* colors,textmode, ...	*/
#include "windows.h"	/* function prototypes	*/

/************************************************
* Screen - save/restore screen (assuming 80x25	*
* color mode).									*
*												*
* INP:	save - save screen if TRUE, restore the	*
*				screen otherwise				*
************************************************/
void	Screen(int save)
{
	static int	row,col;
	static unsigned screen_buffer[25*80];

	if (save) {
		textmode(C80);
		row = wherey();
		col = wherex();
		gettext(1,1,80,25,screen_buffer);
		}
	else {
		window(1,1,80,25);
		normvideo();
		puttext(1,1,80,25,screen_buffer);
		gotoxy(col,row);
		}
}

/************************************************
* Windowputs - write string to current window	*
* at specified coordinates						*
*												*
* INP:	col, row - coordinates to put string	*
*		str - string to write to screen			*
************************************************/
void Windowputs(int col, int row, char *str)
{
	gotoxy(col,row);
	cputs(str);
	gotoxy(col,row);
}

/************************************************
* Windowputch - write character to current		*
* window at specified coordinates				*
*												*
* INP:	col, row - coordinates to put character	*
*		c - character to put on screen			*
************************************************/
void Windowputch(int col, int row, int c)
{
	gotoxy(col,row);
	putch(c);
}

/************************************************
* WindowCreate - create bordered, colored window*
* at specified coordinates.  Active window is	*
* the area inside the borders.					*
*												*
* INP:	left - left side of window				*
*		top    - top side of window				*
*		right  - right side of window			*
*		bottom - bottom side of window			*
*		foreground - color of the letters		*
*		background - color of background		*
************************************************/
void WindowCreate(int left, int top, int right,
	 int bottom, int foreground, int background)
{
	int	i, width, height;

	/* Color in the window */
	window(left,top,right,bottom);
	textcolor(foreground);
	textbackground(background);
	clrscr();

	/* Draw the border */
	height=bottom-top;  width=right-left;
	for (i = 2; i <= height; i++) {
		Windowputs(1,i,"");		/* Left		*/
		Windowputs(width+1,i,"");	/* Right	*/
		}
	for (i = 2; i <= width; i++) {
		Windowputs(i,1,"");		/* Top		*/
		Windowputs(i,height+1,"");	/* Bottom	*/
		}
	Windowputs(1,1,"");				/* UL	*/
	Windowputs(i,1,"");				/* UR	*/
	Windowputs(1,height+1,"");			/* LL	*/
	window(left,top,right+1,bottom);
	Windowputch(width+1,height+1,'');	/* LR	*/

	/* Set active window region size */
	window(left+1,top+1,right-1,bottom-1);
}
